@@ -1,4 +1,4 @@
-# $Id: command.rb,v 1.65 2005/06/29 06:10:45 fdiary Exp $
+# $Id: command.rb,v 1.72 2005/07/14 03:31:31 fdiary Exp $
 # Copyright (C) 2002-2004 TAKEUCHI Hitoshi <hitoshi@namaraii.com>
 
 require 'hiki/page'
@@ -10,6 +10,8 @@ require 'hiki/session'
 include Hiki::Util
 
 module Hiki
+  class PermissionError < StandardError; end
+
   class Command
     def initialize(cgi, db, conf)
       @db     = db
@@ -51,9 +53,9 @@ module Hiki
       options['params'] = @params
 
       @plugin = Plugin::new( options, @conf )
-      session_id = @cgi.cookies['session_id'][0]
-      if session_id
-        @plugin.user = Hiki::Session::new( @conf, session_id ).user
+      @session_id = @cgi.cookies['session_id'][0]
+      if @session_id
+        @plugin.user = Hiki::Session::new( @conf, @session_id ).user
       end
       @body_enter = @plugin.body_enter_proc
     end
@@ -63,10 +65,10 @@ module Hiki
       begin
         raise if !@p && ['view', 'edit', 'diff', 'save'].index( @cmd )
         if @cmd == 'edit'
-          raise 'Permission denied' unless @plugin.auth?
+          raise PermissionError, 'Permission denied' unless @plugin.editable?
           cmd_edit( @p )
         elsif @cmd == 'save'
-          raise 'Permission denied' unless @plugin.auth?
+          raise PermissionError, 'Permission denied' unless @plugin.editable?
           if @params['save'][0]
             cmd_save(@p, @params['contents'][0], @params['md5hex'][0], @params['update_timestamp'][0])
           elsif @params['preview'][0]
@@ -77,7 +79,7 @@ module Hiki
             cmd_edit( @p, @plugin.text )
           end
         elsif @cmd == 'create'  
-          raise 'Permission denied' unless @plugin.auth?
+          raise PermissionError, 'Permission denied' unless @plugin.editable?
           send( "cmd_#{@cmd}" )
         else
           if @conf.use_plugin and @plugin.plugin_command.index(@cmd) and @plugin.respond_to?(@cmd)
@@ -86,8 +88,9 @@ module Hiki
             send( "cmd_#{@cmd}" )
           end
         end
-      rescue NoMethodError
+      rescue NoMethodError, PermissionError
         data = get_common_data( @db, @plugin, @conf )
+        data[:message] = CGI::escapeHTML( $!.message )
         generate_error_page( data )
       end
     end
@@ -134,13 +137,14 @@ module Hiki
     end
 
     def cmd_preview
+      raise PermissionError if @session_id && @session_id != @cgi.params['session_id'][0]
       @cmd = 'preview'
       cmd_edit( @p, @params['contents'][0], @conf.msg_preview, @params['page_title'][0] )
     end
     
     def cmd_view
       if /^\./ =~ @p
-        redirect(@cgi, @conf.cgi_name) 
+        redirect(@cgi, @conf.index_url) 
         return
       end
       unless @db.exist?( @p )
@@ -266,6 +270,11 @@ module Hiki
       link         = nil
       formatter    = nil
       data = get_common_data( @db, @plugin, @conf )
+      if @db.is_frozen?( page ) || @conf.options['freeze']
+        data[:freeze] = ' checked'
+      else
+        data[:freeze] = ''
+      end
 
       if @cmd == 'preview'
         p = @conf.parser::new( @conf ).parse( text.gsub(/\r/, '') )
@@ -274,6 +283,7 @@ module Hiki
         save_button = ''
         data[:keyword] = CGI.escapeHTML( @params['keyword'][0] || '' )
         data[:update_timestamp] = @params['update_timestamp'][0] ? ' checked' : ''
+        data[:freeze] = @params['freeze'][0] ? ' checked' : ''
       elsif @cmd == 'conflict'
         old = text.gsub(/\r/, '')
         new = @db.load( page ) || ''
@@ -308,11 +318,8 @@ module Hiki
       data[:keyword]        ||= CGI.escapeHTML( @db.get_attribute(page, :keyword).join("\n") )
       data[:update_timestamp] ||= ' checked'
       data[:page_title]     = page_title
-      
-      f = @db.is_frozen?( page ) || @conf.options['freeze']
-      data[:freeze]         = f ? ' checked' : ''
-      data[:freeze_msg]     = @conf.msg_freeze if f
       data[:form_proc]      = @plugin.form_proc
+      data[:session_id]     = @session_id || ''
 
       generate_page( data )
     end
@@ -330,6 +337,7 @@ module Hiki
     end
 
     def cmd_save( page, text, md5hex, update_timestamp = true )
+      raise PermissionError if @session_id && @session_id != @cgi.params['session_id'][0]
       subject = ''
       if text.size == 0 && @plugin.admin?
         @db.delete( page )
@@ -340,12 +348,6 @@ module Hiki
         data[:link]      = page.escapeHTML
         generate_page(data)
       else
-        if (@db.is_frozen?( page ) || @conf.options['freeze']) && !@plugin.admin?
-          @cmd = 'edit'
-          cmd_edit( page, text )
-          return
-        end
-
         title = @params['page_title'][0] ? @params['page_title'][0].strip : page
         title = title.size > 0 ? title : page
 
@@ -359,7 +361,7 @@ module Hiki
           keyword = @params['keyword'][0].split("\n").collect {|k|
             k.chomp.strip}.delete_if{|k| k.size == 0}
           attr = [[:keyword, keyword.uniq], [:title, title]]
-          attr << [:editor, @plugin.user] if @plugin.user
+          attr << [:editor, @plugin.user]
           @db.set_attribute(page, attr)
         else
           @cmd = 'conflict'
@@ -368,7 +370,7 @@ module Hiki
         end
 
         @db.freeze_page( page, @params['freeze'][0] ? true : false) if @plugin.admin?
-        redirect(@cgi, @plugin.hiki_url(page))
+        redirect(@cgi, @conf.base_url + @plugin.hiki_url(page))
       end
     end
 
@@ -444,22 +446,20 @@ module Hiki
     def cmd_login
       name = @params['name'][0]
       password = @params['password'][0]
+      page = @params['p'][0]
       msg_login_result = nil
       if name && password
         session = Hiki::Session::new( @conf )
-        user = nil
-        if @conf.password.empty? || password.crypt( @conf.password ) == @conf.password && name == @conf.admin_name
-          user = @conf.admin_name
-        elsif @conf['user.list']
-          if @conf['user.list'].has_key?(name) && @conf['user.list'][name] == password.crypt(@conf['user.list'][name])
-            user = name
-          end
-        end
+        @plugin.login( name, password )
 
-        if user
-          session.user = user
+        if @plugin.user
+          session.user = @plugin.user
           session.save
-          redirect(@cgi, @conf.cgi_name, session_cookie( session.session_id )) 
+          if page && !page.empty?
+            redirect(@cgi, @conf.base_url + @plugin.hiki_url( page ), session_cookie( session.session_id )) 
+          else
+            redirect(@cgi, @conf.index_url, session_cookie( session.session_id )) 
+          end
           return
         else
           msg_login_result = @conf.msg_login_failure
@@ -470,19 +470,22 @@ module Hiki
       data[:title]   = title( @conf.msg_login )
       data[:button]  = @conf.msg_ok
       data[:login_result] = msg_login_result
+      data[:page] = page
       generate_page( data )
     end
 
     def cmd_admin
-      unless @plugin.admin?
-        redirect(@cgi, @conf.cgi_name)
-        return
-      end
+      raise PermissionError, 'Permission denied' unless @plugin.admin?
+
       data = get_common_data( @db, @plugin, @conf )
       data[:key]            = @cgi.params['conf'][0] || 'default'
 
       data[:title]          = title( @conf.msg_admin )
-      data[:save_config]    = true if @cgi.params['saveconf'][0]
+      data[:session_id]     = @session_id
+      if @cgi.params['saveconf'][0]
+        raise PermissionError if @session_id && @session_id != @cgi.params['session_id'][0]
+        data[:save_config]    = true
+      end
       generate_page( data )
     end
 
@@ -521,7 +524,7 @@ module Hiki
       end
 
       if redirect_mode and result
-        redirect(@cgi, @plugin.hiki_url(@p))
+        redirect(@cgi, @conf.base_url + @plugin.hiki_url(@p))
       end
     end
 
@@ -530,7 +533,7 @@ module Hiki
         cookies = [session_cookie(session_id, -1)]
         Hiki::Session::new( @conf, session_id ).delete
       end
-      redirect(@cgi, @conf.cgi_name, cookies)
+      redirect(@cgi, @conf.index_url, cookies)
     end
 
     def cookie(name, value, max_age = Session::MAX_AGE)
