@@ -1,10 +1,10 @@
-# $Id: config.rb,v 1.65 2005/06/29 06:20:09 fdiary Exp $
+# $Id: config.rb,v 1.71 2005/07/14 03:31:31 fdiary Exp $
 # Copyright (C) 2004-2005 Kazuhiko <kazuhiko@fdiary.net>
 #
 # TADA Tadashi <sho@spc.gr.jp> holds the copyright of Config class.
 
-HIKI_VERSION  = '0.8.0'
-HIKI_RELEASE_DATE = '2005-06-29'
+HIKI_VERSION  = '0.8.1'
+HIKI_RELEASE_DATE = '2005-07-14'
 
 require 'cgi'
 require 'hiki/command'
@@ -17,10 +17,11 @@ module Hiki
       load
       load_cgi_conf
 
+      load_messages
+
       require "style/#{@style}/parser"
       require "style/#{@style}/html_formatter"
       require "hiki/repos/#{@repos_type}"
-      require "messages/#{@lang}"
       require "hiki/db/#{@database_type}"
 
       # parser class and formatter class
@@ -28,9 +29,6 @@ module Hiki
       @parser = Hiki::const_get( "Parser_#{style}" )
       @formatter = Hiki::const_get( "HTMLFormatter_#{style}" )
 
-      # message module
-      extend(Hiki::const_get( "Messages_#{@lang}" ))
-
       # repository class
       @repos = Hiki::const_get("Repos#{@repos_type.capitalize}").new(@repos_root, @data_path)
 
@@ -148,7 +146,6 @@ module Hiki
       @hilight_keys  ||= true
       @plugin_debug  ||= false
       @charset       ||= 'EUC-JP'
-      @lang          ||= 'ja'
       @database_type ||= 'flatfile'
       @cgi_name      ||= './'
       @admin_name    ||= 'admin'
@@ -239,6 +236,38 @@ module Hiki
       nil
     end
 
+    def load_messages
+      candidates = @lang ? [@lang] : []
+
+      if ENV['HTTP_ACCEPT_LANGUAGE']
+        accept_language = ENV['HTTP_ACCEPT_LANGUAGE'].split(',').collect{|entry|
+          lang, quality = entry.split(';')
+          lang.strip!
+          if /^q=(.+)/ =~ quality
+            quality = $1.to_f
+          else
+            quality = 1.0
+          end
+          [lang, quality]
+        }.sort_by{|i| -i[1]}.map{|i| i[0][0...2].untaint}
+
+        candidates.concat(accept_language)
+      end
+
+      candidates << 'en'
+
+      candidates.each do |lang|
+        begin
+          require "messages/#{lang}"
+          extend(Hiki::const_get("Messages_#{lang}"))
+          @lang = lang
+          return
+        rescue LoadError
+        end
+      end
+      raise "No message resource file is found. Please put one in the messages/ directory."
+    end
+
     def formaterror
       raise "*** NOTICE ***\n\nThe format of configuration files (i.e. hikiconf.rb and hiki.conf) has changed.\nSee 'doc/VERSIONUP.txt' for more details.\n\n"
     end
