@@ -1,4 +1,4 @@
-# $Id: attach.rb,v 1.22 2005/06/17 06:37:59 fdiary Exp $
+# $Id: attach.rb,v 1.24 2005/07/12 08:32:01 fdiary Exp $
 # Copyright (C) 2003 TAKEUCHI Hitoshi <hitoshi@namaraii.com>
 #
 # thanks to Kazuhiko, Masao Mutoh, SHIMADA Mitsunobu, Yoshimi, ꤿ
@@ -34,7 +34,7 @@ def attach_map
   attach_files.sort do |a, b|
     a[0].unescape <=> b[0].unescape
   end.each do |attach_info|
-    s << "<li>#{hiki_anchor(attach_info[0], page_name(attach_info[0]))}\n"
+    s << "<li>#{hiki_anchor(attach_info[0], page_name(attach_info[0].unescape))}\n"
     s << "<ul>\n"
     attach_info[1].each do |f|
       s << "<li>#{attach_anchor(f, attach_info[0].unescape)}\n"
@@ -224,4 +224,4 @@ add_form_proc {
   end
 }
 
-export_plugin_methods(:attach_map, :attach_anchor_string, :attach_anchor, :attach_image_anchor, :attach_flash_anchor, :attach_download, :attach_src, :attach_view)
+export_plugin_methods(:attach_form, :attach_map, :attach_anchor_string, :attach_anchor, :attach_image_anchor, :attach_flash_anchor, :attach_download, :attach_src, :attach_view)
