@@ -1,4 +1,4 @@
-# $Id: html_formatter.rb,v 1.33 2005/06/24 09:14:33 fdiary Exp $
+# $Id: html_formatter.rb,v 1.39 2005/07/13 01:43:06 fdiary Exp $
 # Copyright (C) 2002-2003 TAKEUCHI Hitoshi <hitoshi@namaraii.com>
 
 require 'hiki/util'
@@ -11,7 +11,7 @@ module Hiki
     
   class HTMLFormatter_default < HikiFormatter
     MAP = Hash::new
-    MAP[:heading1_open]        = '<h2><span class="date">'
+    MAP[:heading1_open]        = %Q!\n<h2><span class="date">!
     MAP[:heading1_open_end]    = '</span><span class="title">'
     MAP[:heading1_close]       = '</span></h2>'
     MAP[:heading2_open]        = '<h3>'
@@ -119,7 +119,6 @@ module Hiki
         s << %Q!#{map(:listitem_open)}<a href="#l#{h['index']}">#{h['title'].escapeHTML}</a>#{map(:listitem_close)}\n!
       end
       s << ("#{map(:unordered_list_close)}\n" * lv)
-      s = %Q!<div class="day"><div class="body"><div class="section">#{s}</div></div></div>! unless @conf.mobile_agent?
       s
     end
     
@@ -190,11 +189,7 @@ module Hiki
             s[:html] << str
           end
         rescue Exception => e
-          if @conf.plugin_debug
-            s[:html] << e.message
-          else
-            s[:html] << plugin_error( t[:method], $! )
-          end
+          s[:html] << e.message
         end
       when :table_head_open, :table_data_open
         rws = ''
@@ -205,7 +200,7 @@ module Hiki
           rws = %Q| rowspan="#{t[:row]}"|
         end
         if t[:col] > 1
-          cls = %Q| colspan=#{t[:col]}"|
+          cls = %Q| colspan="#{t[:col]}"|
         end
         str = mp[0,len-1]
         str << rws
@@ -233,12 +228,15 @@ module Hiki
     end
 
     def tdiary_section(title, section)
+      title.strip!
+      section.strip!
+      return '' if title.empty? && section.empty?
 <<"EOS"
 <div class="day">
-  #{title.chomp}
+  #{title}
   <div class="body">
     <div class="section">
-      #{section.chomp}
+      #{section}
     </div>
   </div>
 </div>
@@ -303,7 +301,7 @@ EOS
           s[:html] << @plugin.make_anchor(outer_alias[0], outer_alias[1], 'external')
         else
           missing_anchor_title = @conf.msg_missing_anchor_title % [ disp.escapeHTML ]
-          wikiname_anchor = @plugin.auth? ? "#{disp.escapeHTML}<a class=\"nodisp\" href=\"#{@conf.cgi_name}?c=edit;p=#{t[:href].escape}\" title=\"#{missing_anchor_title}\">?</a>" : disp.escapeHTML
+          wikiname_anchor = @plugin.creatable? ? "#{disp.escapeHTML}<a class=\"nodisp\" href=\"#{@conf.cgi_name}?c=edit;p=#{t[:href].escape}\" title=\"#{missing_anchor_title}\">?</a>" : disp.escapeHTML
           s[:html] << wikiname_anchor
         end
       end
